/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <unistd.h>
#include <sys/time.h>

#include <qmap.h>

#include <kapplication.h>
#include <kdebug.h>

#include "computationdata.h"
#include "packingdata.h"
#include "algobase.h"

algobase::algobase() 
{
    m_data = packingdata::getData();
    m_cancompute = false;
}

algobase::~algobase() 
{
}

void algobase::start()
{
    m_data->m_computing = true;
    m_cancompute = true;

    struct timeval start_time;
    gettimeofday( &start_time, NULL );
    m_usec_start = start_time.tv_usec;
    m_sec_start = start_time.tv_sec;
}

void algobase::stop()
{
    m_cancompute = false;
}

void algobase::updateAlgoResults()
{
    updateTotalTime();

    m_data->m_cdata.m_arearatio = m_data->ratioArea();
    m_data->m_cdata.m_height = m_data->Haut();
    m_data->m_cdata.m_containerLB = m_data->LB();
    m_data->m_cdata.m_nbrect = m_data->nbValidRect();
}

void algobase::updateTotalTime()
{
    struct timeval end_time;
    gettimeofday(&end_time, NULL);
    int sectime = end_time.tv_sec  - m_sec_start;
    int usectime = end_time.tv_usec - m_usec_start;
    float totaltime=0;

    if (usectime > 0)
	totaltime = (1000000*(float)sectime + (float)usectime )/1000;
    else
	totaltime = (1000000*(float)sectime - (float)usectime)/1000;

    m_data->m_cdata.m_totaltime = (int) (totaltime/1000.0);
}

void algobase::loadSettings()
{
    setParams();
    
    if (!m_data)
	return;

    // use the default settings
    m_settings = m_defaultsettings;

    // try to use the stored settings, if there are
    KConfig *cfg = new KConfig("linpackerrc");

    QMap< QString, QString > map = cfg->entryMap( name() );
    if (!map.isEmpty() )
    {
	QValueList<QString> lst = map.keys();
	for (int i=0; i<lst.count(); i++)
	{
	    QString val = lst[i];
	    
	    if (val == "iterations")
		m_settings.m_iter        = map[val].toInt();
	    else if (val == "population")
		m_settings.m_population  = map[val].toInt();
	    else if (val == "mutations")
		m_settings.m_mutations   = map[val].toInt();
	    else if (val == "crossovers")
		m_settings.m_crossovers  = map[val].toInt();
	    else if (val == "maxtime")
		m_settings.m_maxtime     = map[val].toInt();
	    else if (val == "fitnesstype")
		m_settings.m_fitnesstype = map[val].toInt();
	}
    }

    delete cfg;
}

void algobase::saveSettings()
{
    if (!m_data)
	return;

    KConfig *cfg = new KConfig("linpackerrc");
    cfg->setGroup( name() );

    cfg->writeEntry("iterations", m_settings.m_iter);
    cfg->writeEntry("population", m_settings.m_population);
    cfg->writeEntry("mutations", m_settings.m_mutations);
    cfg->writeEntry("crossovers", m_settings.m_crossovers);
    cfg->writeEntry("maxtime", m_settings.m_maxtime);
    cfg->writeEntry("fitnesstype", m_settings.m_fitnesstype);

    delete cfg;
}

void algobase::loadDefaultSettings()
{
    m_settings = m_defaultsettings;
}



