/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _LINPACKERALGOBASE_H
#define _LINPACKERALGOBASE_H

#include <qstring.h>

#include <kconfig.h>

class algodata;
class packingdata;
class algobase
{
    public:

	algobase();
	virtual ~algobase();

	// Note: use algobase::start and algobase::stop 
	// in the start() method of subclasses
	virtual void start();
	virtual void stop();

	// TODO is currentHeight really needed ?
	// we update the data everytime a better solution is found
	virtual int currentHeight() =0;
	virtual void cleanup() = 0;
	virtual QString name() = 0;
	virtual QString description() = 0;

	virtual void setParams()=0;

	void loadSettings();
	void saveSettings();
	void loadDefaultSettings();
	
	// TODO : make an accessor for m_settings
	// in case if there are wrong parameters
	algodata m_settings;
	algodata m_tmpsettings;
	algodata m_defaultsettings;
	
	void updateAlgoResults();
	void updateTotalTime();

    protected:
	packingdata * m_data;
	bool m_cancompute;

    private:
	int m_usec_start;
	int m_sec_start;
};

#endif // _LINPACKERALGOBASE_H

