/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <kglobal.h>
#include <kstandarddirs.h>

#include <qprocess.h>
#include <qdir.h>
#include <qstringlist.h>

#include "computationdata.h"
#include "packingdata.h"
#include "strip.h"
#include "algoproc.h"

#include "algobib.h"

static algobib *bib = NULL;
algobib* algobib::getBib()
{
    if (bib == NULL)
	bib = new algobib();
    return bib;
}

algobib::algobib() 
{
    m_data = packingdata::getData();
    initAlgorithms();
}

algobib::~algobib() 
{
    clearAlgo();
}

void algobib::clearAlgo()
{
    for (unsigned int i=0; i< m_biblio.size(); i++)
	delete m_biblio[i];
}

void algobib::initAlgorithms()
{
    clearAlgo();

    QProcess *proc = new QProcess;

    // the following doesn't work
    //kdWarning()<<"trying .. :"<<KGlobal::dirs()->findResourceDir("appdata", "plugins/pluginlist.doc")<<endl;
    
    QString docname = "pluginlist.doc";
    QString plugindocpath = locate( "appdata", "plugins/"+docname );
    if (plugindocpath.endsWith(docname))
    {
	QString plugindir = plugindocpath.left( plugindocpath.length() - docname.length() );
	QDir dir(plugindir);
	QStringList filelist = dir.entryList( QDir::Files|QDir::Executable );
	
	QProcess proc;
	for (int i=0; i<filelist.count(); i++)
	{
	    proc.clearArguments();
	    proc.addArgument( plugindir+filelist[i] );
	    proc.addArgument( "-a" );

	    proc.start();

	    // give up if it takes more than 2 seconds
	    int timespent = 0;
	    while (!proc.canReadLineStdout() && timespent < 2000000)
	    {
		KApplication::kApplication()->processEvents();
		usleep(250);
		timespent+=250;
	    }
	    
	    if (!proc.canReadLineStdout())
	    {
		proc.kill();
		continue;
	    }

	    QString idline = proc.readLineStdout();
	    QStringList idlist = QStringList::split( "\t", idline );
	    
	    for (int j=0;j<idlist.count();j++)
	    {
		m_biblio.push_back( new algoproc(filelist[i], idlist[j].toInt() ) );
	    }

	    // make sure it is really dead :)
	    proc.kill();
	}
    }

    // add the other solvers
    m_biblio.push_back(new Strip());
}

algobase* algobib::defaultAlgo()
{
    if (m_biblio.size() > 0)
	return m_biblio[0];
    return NULL;
}

algobase* algobib::getAlgo(QString algoname)
{
    algobase* returnalgo=NULL;
    for (unsigned int i=0; i<m_biblio.size(); i++)
    {
	if (m_biblio[i]->name() == algoname)
	{
	    returnalgo = m_biblio[i];
	    break;
	}
    }
    return returnalgo;
}

QStringList algobib::algorithms_available()
{
    QStringList strlist;
    for (unsigned int i=0; i<m_biblio.size(); i++)
    {
	strlist.append(m_biblio[i]->name());
    }
    return strlist;
}

void algobib::makeTmpSettings()
{
    for (unsigned int i=0; i< m_biblio.size(); i++)
    {
	m_biblio[i]->m_tmpsettings = m_biblio[i]->m_settings;
    }
}

void algobib::loadTmpSettings()
{
    for (unsigned int i=0; i< m_biblio.size(); i++)
    {
	m_biblio[i]->m_settings = m_biblio[i]->m_tmpsettings;
	m_biblio[i]->saveSettings();
    }
}

