/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kprogress.h>
#include <klocale.h>
#include <kdebug.h>
#include <kapplication.h>

#include "algobib.h"
#include "../computationdata.h"
#include "../packingdata.h"
#include "algobase.h"
#include "algorun.h"
#include "settings.h"

class algothread : public QThread
{
    public:
	virtual void run();
	void setAlgo(algobase*);

    private:
	algobase *m_algo;
};

void algothread::run()
{
    // DEBUG
    //kdWarning()<<"thread started"<<endl;
    if (m_algo)
	m_algo->start();
}

void algothread::setAlgo(algobase* algo)
{
    if (!algo)
	kdWarning()<<"uh oh .."<<endl;
    m_algo = algo;
}

// see below
static algorun* s_algorun = NULL;

// static method, returns the runner linpacker needs
algorun* algorun::getRunner()
{
    if (!s_algorun)
	s_algorun = new algorun();
    return s_algorun;
}

algorun::algorun()
{
    m_data = packingdata::getData();
    m_algobib = algobib::getBib();
}

algorun::~algorun()
{
}

void algorun::start()
{
    if (!m_data->pbvalide())
	return;
	
    m_data->m_ischanged=true;
    
    KProgressDialog *pr = new KProgressDialog(NULL, NULL,
	    i18n("Computing a solution"), i18n("Searching ..."), true);

    pr->setButtonText( i18n("Stop") );

    // it is likely to last for some time,
    // popup the progressdialog after 1 second
    pr->setMinimumDuration( 1000 ); 

    algothread *thr = new algothread();
    algobase *algo = m_algobib->getAlgo( m_data->m_adata.m_algoname );

    if (!algo)
	kdWarning()<<"using a null algorithm (BUG) - linpacker will probably crash .."<<endl;
    
    thr->setAlgo(algo);
    thr->start();

    KProgress *progress = pr->progressBar();

    int count = 0;
    int progressval = 0;
    progress->setProgress(0);
    while (thr->running())
    {
	count++;

	/* update the gui every 300 milliseconds */
	usleep( Settings::guiupdate()*1000 );

	/* stop requested by user */
	if (pr->wasCancelled())
	{
	    m_data->m_cdata.m_textlog += i18n("Stop requested\n");
	    algo->stop();
	    break;
	}

	/* compute the time taken .. */
	algo->updateTotalTime();

	/* update the progress dialog */
	
	// use the algorithm status
	int newprogressval = m_data->m_status;

	// use the time limit, if it is set
	if (m_data->m_adata.m_maxtime > 0 )
	{
	    // TODO isn't this formula is a bit complicated ?
	    int timeprogress = (int) (((double) (100.0*m_data->m_cdata.m_totaltime))/m_data->m_adata.m_maxtime);
	    newprogressval = QMAX(newprogressval, timeprogress );
	}

	if (newprogressval > progressval)
	{
	    if (newprogressval >= 100)
		newprogressval=99;

	    progressval = newprogressval;
	    progress->setProgress(progressval);
	}
	
	/* the values m_data->m_cdata* are updated by the algo thread */
	QString totaltime = i18n("total time : %1").arg(m_data->m_cdata.m_totaltime);
	QString totalheight = i18n("total height : %1").arg(m_data->m_cdata.m_height);
	QString areacovered = i18n("area covered : %1").arg(m_data->m_cdata.m_arearatio);
	
	pr->setLabel( totaltime+"\n"+totalheight+"\n"+areacovered );

	KApplication::kApplication()->processEvents();

	/* check against the time limit */
	if (m_data->m_adata.m_maxtime > 0 )
	    if (m_data->m_cdata.m_totaltime >= m_data->m_adata.m_maxtime)
		algo->stop();
    }

    // the thread must have stopped - wait a bit to make sure it has
    while (thr->running())
    {
	count++;

	/* update the gui every 30 milliseconds */
	usleep(30000);
	KApplication::kApplication()->processEvents();
    }

    // now it does have finished
    progress->setProgress(100);

    m_data->m_cdata.m_textlog += i18n("** %1 : success **\n").arg( algo->name() );
    
    delete thr;
}


