/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>

#include "rectangle.h"

rectangle::rectangle(int lo, int la, int xpos, int ypos,bool orient,int num)
{
    r_long = lo;
    r_larg = la;
    posx = xpos;
    posy = ypos;
    orientation = orient;
    numero = num;
}

rectangle::rectangle()
{
    r_long = 0;
    r_larg = 0;
    posx = 0;
    posy = 0;
    orientation = 0;
    numero = 0;
}

// static
bool rectangle::overlapping(rectangle a, rectangle b)
{
    if ( QABS( 2*(a.posx - b.posx) + (a.largeur() - b.largeur())  ) < (a.largeur() + b.largeur()) )
    	if ( QABS( 2*(a.posy - b.posy) + (a.longueur() - b.longueur())  ) < (a.longueur() + b.longueur()) )
    	    return true;
	
#ifndef DEBUG
    kdWarning()<<a.identifikation()<<endl;
    kdWarning()<<b.identifikation()<<endl;
    kdWarning()<<"not overlapping"<<endl;
#endif
    return false;
}

QString rectangle::identifikation()
{
    QString val = i18n("W : ");
    val += QString::number(r_larg);
    val += " | ";

    val += i18n("H : ");
    val += QString::number(r_long);
    val += " | ";

    val += i18n("Px : ");
    val += QString::number(posx);
    val += " | ";

    val += i18n("Py : ");
    val += QString::number(posy);
    val += " | ";

    val += i18n("Rotated : ");
    if (orientation)
	val += "true";
    else
	val += "false";

    val += " | ";
    val += i18n("Id : ");
    val += QString::number(numero);

    return val;
}

int rectangle::longueur() const
{
    if (orientation)
	return r_long;	
    return r_larg;
}

int rectangle::largeur() const
{
    if (orientation)
	return r_larg;	
    return r_long;
}

int rectangle::YHaut() const
{
    return (posy + longueur());	
}

int rectangle::XMax() const
{
    return (posx + largeur());
}

int rectangle::Surface() const
{
    return (largeur() * longueur());
}

void rectangle::print() const
{
    kdDebug() <<"rectangle "<<
	r_long<<" "<<
	r_larg<<" "<<
	posx<<" "<<
	posy<<" "<<
	orientation<<" "<<
	numero<<endl;
}

lieu::lieu()
{
    posx=0;
    posy=0;
}

lieu::lieu(int i, int j)
{
    posx=i;
    posy=j;
}

bool operator< (rectangle x, rectangle y)
{
    return (x.Surface() > y.Surface());
}

bool operator< (lieu x, lieu y)
{
    if ( (x.posy < y.posy) ) //or (x.posy == y.posy and x.posx <y.posx))
	return 1;
    return 0;
}

