/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qcombobox.h>
#include <qslider.h>
#include <qtextedit.h>
#include <qspinbox.h>

#include <kdebug.h>

#include "computationdata.h"
#include "packingdata.h"
#include "algo/algobase.h"
#include "algo/algobib.h"
#include "algo_chooser.h"

algo_chooser::algo_chooser(QWidget*parent) : algo_base(parent)
{
    QStringList lst = algobib::getBib()->algorithms_available();
    QString prev_algo_name = packingdata::getData()->m_adata.m_algoname;

    int idx = -1;
    for (int i=0; i<lst.count(); i++)
    {
	if (lst[i] == prev_algo_name)
	{
	    idx = i;
	    break;
	}
    }
    
    algoname->insertStringList( lst );
    connect(algoname, SIGNAL(activated(const QString &)), this, SLOT(change_algo(const QString &)));
    
    connect(mutation, SIGNAL(valueChanged(int)), this, SLOT(change_mutation(int)));
    connect(crossover, SIGNAL(valueChanged(int)), this, SLOT(change_crossover(int)));
    connect(iterations, SIGNAL(valueChanged(int)), this, SLOT(change_iteration(int)));
    connect(maxtime, SIGNAL(valueChanged(int)), this, SLOT(change_maxtime(int)));
    connect(population, SIGNAL(valueChanged(int)), this, SLOT(change_population(int)));

    if (idx != -1)
	algoname->setCurrentItem( idx );
	
    change_algo( algoname->currentText() );
}

algo_chooser::~algo_chooser()
{

}

void algo_chooser::change_algo(const QString & str)
{
    //kdWarning()<<"algo_chooser::change_algo"<<endl;
    
    algobase * algo = algobib::getBib()->getAlgo(str);

    if (!algo)
    {
	kdWarning()<<"algorithm chosen does not exist anymore"<<endl;
	return;
    }
    
    algodescr->setText( algo->description() );

    m_adata.m_algoname = algo->name();
    
    algodata settings = algo->m_settings;
    
    mutation->setValue( settings.m_mutations );
    crossover->setValue( settings.m_crossovers );
    iterations->setValue( settings.m_iter );
    maxtime->setValue( settings.m_maxtime );
    population->setValue( settings.m_population );
}

void algo_chooser::change_mutation(int val)
{
    //kdWarning()<<"algo_chooser::change_mutation"<<endl;
    m_adata.m_mutations = val;
}

void algo_chooser::change_crossover(int val)
{
    //kdWarning()<<"algo_chooser::change_crossover"<<endl;
    m_adata.m_crossovers = val;
}

void algo_chooser::change_iteration(int val)
{
    //kdWarning()<<"algo_chooser::change_iteration"<<endl;
    m_adata.m_iter = val;
}

void algo_chooser::change_maxtime(int val)
{
    //kdWarning()<<"algo_chooser::change_maxtime"<<endl;
    m_adata.m_maxtime = val;
}

void algo_chooser::change_population(int val)
{
    //kdWarning()<<"algo_chooser::change_population"<<endl;
    m_adata.m_population = val;
}

#include "algo_chooser.moc"

