/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qcombobox.h>
#include <qslider.h>
#include <qtextedit.h>
#include <qspinbox.h>

#include <kdebug.h>

#include "computationdata.h"
#include "algo/algobase.h"
#include "algo/algobib.h"
#include "algo_prefs.h"

algo_prefs::algo_prefs(QWidget*parent) : algo_base(parent)
{
    algoname->insertStringList( algobib::getBib()->algorithms_available() );

    connect(algoname, SIGNAL(activated(const QString &)), this, SLOT(change_algo(const QString &)));
    connect(mutation, SIGNAL(valueChanged(int)), this, SLOT(change_mutation(int)));
    connect(crossover, SIGNAL(valueChanged(int)), this, SLOT(change_crossover(int)));
    connect(iterations, SIGNAL(valueChanged(int)), this, SLOT(change_iteration(int)));
    connect(maxtime, SIGNAL(valueChanged(int)), this, SLOT(change_maxtime(int)));
    connect(population, SIGNAL(valueChanged(int)), this, SLOT(change_population(int)));

    change_algo( algoname->currentText() );
}

algo_prefs::~algo_prefs()
{

}

void algo_prefs::change_algo(const QString & str)
{
    //kdWarning()<<"algo_prefs::change_algo"<<endl;

    algobase * algo = algobib::getBib()->getAlgo(str);
    algodescr->setText( algo->description() );

    algodata settings = algo->m_tmpsettings;
    
    mutation->setValue( settings.m_mutations );
    crossover->setValue( settings.m_crossovers );
    iterations->setValue( settings.m_iter );
    maxtime->setValue( settings.m_maxtime );
    population->setValue( settings.m_population );
}

void algo_prefs::change_mutation(int val)
{
    //kdWarning()<<"algo_prefs::change_mutation"<<endl;
    algobib::getBib()->getAlgo( algoname->currentText() )->m_tmpsettings.m_mutations = val;
}

void algo_prefs::change_crossover(int val)
{
    //kdWarning()<<"algo_prefs::change_crossover"<<endl;
    algobib::getBib()->getAlgo( algoname->currentText() )->m_tmpsettings.m_crossovers = val;
}

void algo_prefs::change_iteration(int val)
{
    //kdWarning()<<"algo_prefs::change_iteration"<<endl;
    algobib::getBib()->getAlgo( algoname->currentText() )->m_tmpsettings.m_iter = val;
}

void algo_prefs::change_maxtime(int val)
{
    //kdWarning()<<"algo_prefs::change_maxtime"<<endl;
    algobib::getBib()->getAlgo( algoname->currentText() )->m_tmpsettings.m_maxtime = val;
}

void algo_prefs::change_population(int val)
{
    //kdWarning()<<"algo_prefs::change_population"<<endl;
    algobib::getBib()->getAlgo( algoname->currentText() )->m_tmpsettings.m_population = val;
}

#include "algo_prefs.moc"

