/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qmutex.h>
#include <qframe.h>
#include <qlayout.h>
#include <qtable.h>
#include <qspinbox.h>

#include <klocale.h>

#include "computationdata.h"
#include "packingdata.h"
#include "boxmanager_content.h"
#include "boxmanager.h"


boxmanager::boxmanager( QWidget*parent, const char * name, packingdata *tbl) : 
KDialogBase(parent, name, true, i18n("Change the boxes used in the packing"), Ok|Apply|Cancel)
{
    m_data = tbl;
    m_content = new boxmanager_content(this);
    setMainWidget(m_content);

    loadData();
    
    connect(this, SIGNAL(applyClicked()), this, SLOT(slotApply()));
    connect(this, SIGNAL(okClicked()), this, SLOT(slotApply()));
}

boxmanager::~boxmanager()
{
    delete m_content;
}

void boxmanager::slotApply()
{
    for (int i=0;i<m_data->nbRect();i++)
    {
	m_data->setWidth(i, m_content->rtable->text(i,0).toInt() );
	m_data->setHeight(i, m_content->rtable->text(i,1).toInt() );
    }
    m_data->setLB( m_content->spinWidth->value() );
    m_data->makePackingValid();

    loadData();
}

void boxmanager::loadData()
{
    m_content->rtable->setColumnReadOnly(0, false);
    m_content->rtable->setColumnReadOnly(1, false);

    // display data
    m_content->spinWidth->setValue( m_data->LB() );
    m_content->rtable->setNumRows( m_data->nbRect() );
    for (int i=0;i<m_data->nbRect();i++)
    {
	m_content->rtable->setText(i, 0, QString::number( m_data->width(i) ) );
	m_content->rtable->setText(i, 1, QString::number( m_data->height(i) ) );
    }


}

#include "boxmanager.moc"


