/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <math.h>

#include <stdlib.h>

#include <qpainter.h>

#include <kdebug.h>

#include "settings.h"
#include "canvas_rect.h"

#include "constants.h"

bool canvas_rect::s_dofish = true;
QPoint canvas_rect::defCenter = QPoint(0, 0);

    canvas_rect::canvas_rect( QCanvas * cv) :
QCanvasPolygon( cv )
{
    setDims(10, 10, 0, 0);

    m_rectpoints = QPointArray(5);
    m_fishpoints = QPointArray(200);

    m_selected = false;
    setFish(false);

    m_textlabel = new QCanvasText( cv );

    setVisibleLabel( false );

    setZ(5);
    m_textlabel->setZ(6);
}

canvas_rect::~canvas_rect()
{
    delete m_textlabel;
}

void canvas_rect::setVisibleLabel( bool yes )
{
    if (yes)
	m_textlabel->show();
    else
	m_textlabel->hide();
}

void canvas_rect::colorer(QColor c)
{
    m_couleur = c;
    setBrush(c);
}

bool canvas_rect::click()
{
    m_selected = ! m_selected;
    repeindre();
    return m_selected;
}

void canvas_rect::repeindre()
{
    if (m_selected)
	setBrush(m_couleur.light());
    else
	setBrush(m_couleur);
}

void canvas_rect::setDims(float w, float h, float x, float y)
{
    m_x = x;
    m_y = y;
    m_width = w;
    m_height = h;
}

void canvas_rect::drawShape(QPainter &p)
{
    QRect r = boundingRect();
    QRect rtext = m_textlabel->boundingRect();
    m_textlabel->move(r.left() + r.width()/2 - rtext.width()/2, 
	    r.top() + r.height()/2 - rtext.height()/2);


    //QCanvasPolygon::drawShape(p);

    p.setPen(QColor(0,0,255));
    p.setPen(Qt::SolidLine);


    QPointArray pa = QPointArray( areaPoints() );
    pa.detach();
    
    if (pa.count() == 5 )
    {
	pa[0] += QPoint(0,0);
	pa[1] += QPoint(-1,0);
	pa[2] += QPoint(-1,-1);
	pa[3] += QPoint(0,-1);
	pa[4] += QPoint(0,0);
    }
    else
    {
	// remove the invisible border (see below)
	int pts = pa.count()/4;
	for (int i=0; i<pts; i++)
	{
	    pa[i]       += QPoint(0, 12);
	    pa[i+pts]   += QPoint(-12, 0);
	    pa[i+2*pts] += QPoint(0, -12);
	    pa[i+3*pts] += QPoint(12, 0);
	}
    }
    p.drawPolygon( pa );
    p.drawPolyline( pa );
}

void canvas_rect::setFish(bool b)
{
    invalidate();
    m_fish = b;

    if ( !s_dofish || !m_fish || Settings::fisheye()==Settings::EnumFisheye::NoFisheye)
    {
	m_rectpoints[0] = QPoint( (int) m_x, (int) m_y );
	m_rectpoints[1] = QPoint( (int) (m_x+m_width), (int) m_y );
	m_rectpoints[2] = QPoint( (int) (m_x+m_width), (int) (m_y+m_height) );
	m_rectpoints[3] = QPoint( (int) m_x, (int) (m_y+m_height) );
	m_rectpoints[4] = QPoint( (int) m_x, (int) m_y );

	setPoints(m_rectpoints);
    }
    else
    {
	//float sizemax = (m_width>m_height)?m_width:m_height;
	//int pts = (int) (sizemax/15) + 4;
	int pts = 200/4;
	float div = pts - 1;

	//QPointArray pa = QPointArray(4*pts);

	if (Settings::fisheye() == Settings::EnumFisheye::Square )
	{
	    for (int i=0; i<pts; i++)
	    {
		m_fishpoints[i] = fisheyeC( (m_x + (m_width*i)/div), m_y );
		m_fishpoints[i+pts] = fisheyeC( (m_x + m_width), (m_y + (m_height*i)/div));
		m_fishpoints[i+2*pts] = fisheyeC( (m_x + m_width - (m_width*i)/div), (m_y + m_height) );
		m_fishpoints[i+3*pts] = fisheyeC( (m_x), (m_y + m_height - (m_height*i)/div));
	    }
	}
	else if (Settings::fisheye() == Settings::EnumFisheye::Sphere )
	{
	    for (int i=0; i<pts; i++)
	    {
		m_fishpoints[i] = fisheyeS( (m_x + (m_width*i)/div), m_y );
		m_fishpoints[i+pts] = fisheyeS( (m_x + m_width), (m_y + (m_height*i)/div));
		m_fishpoints[i+2*pts] = fisheyeS( (m_x + m_width - (m_width*i)/div), (m_y + m_height) );
		m_fishpoints[i+3*pts] = fisheyeS( (m_x), (m_y + m_height - (m_height*i)/div));
	    }
	}

	// This sets an invisible border around the object
	// to avoid artefacts (trailing black pixels while moving
	// the fisheye lens)
	// 
	// QCanvasPolygon do not have outlines (yet)
	//
	for (int i=0; i<pts; i++)
	{
	    m_fishpoints[i]       += QPoint(0, -12);
	    m_fishpoints[i+pts]   += QPoint(12 - 1, 0);
	    m_fishpoints[i+2*pts] += QPoint(0, 12 - 1);
	    m_fishpoints[i+3*pts] += QPoint(-12, 0);
	}

	setPoints( m_fishpoints );
    }
}

float FDeform(float var)
{
    return ( ( _R_ * var * (_D_+1) ) / (var*_D_ + _R_) ) / var;
}

// spherical fisheye
QPoint canvas_rect::fisheyeS(float x, float y)
{
    float x_ = x - (float) canvas_rect::defCenter.x();
    float y_ = y - (float) canvas_rect::defCenter.y();
    float dist = sqrt(x_*x_ + y_*y_);

    if (dist >= _R_)
	return QPoint((int) x, (int) y);

    //float coeff = FDeform(dist);
    float coeff = ( ( _R_ * dist * (_D_+1) ) / (dist*_D_ + _R_) ) / dist;


    if ( coeff < -30000) /*float::nan*/
    {
	coeff = 1;
    }

    return  QPoint( canvas_rect::defCenter.x() + (int) (x_*coeff),
	    canvas_rect::defCenter.y() + (int) (y_*coeff) );
}

// cartesian fisheye
QPoint canvas_rect::fisheyeC(float x, float y)
{
    float distx = x - (float) canvas_rect::defCenter.x();
    float disty = y - (float) canvas_rect::defCenter.y();

    if ( QABS(distx) > _R_ || QABS(disty) > _R_)
	return QPoint((int) x, (int) y);

    float coeffx = FDeform(QABS(distx));
    float coeffy = FDeform(QABS(disty));

    if ( coeffx < - 30000 )
    {
	coeffx =1;
    }

    if ( coeffy < - 30000 )
    {
	coeffy =1;
    }

    return QPoint( canvas_rect::defCenter.x() + (int) (distx*coeffx),
	    canvas_rect::defCenter.y() + (int) (disty*coeffy) );
}

void canvas_rect::setDefCenter(QPoint p)
{
    canvas_rect::defCenter.setX( p.x() );
    canvas_rect::defCenter.setY( p.y() );
}

void canvas_rect::checkFish()
{
    invalidate();

    // TODO ? : better test

    // quick test
    if ( m_x > canvas_rect::defCenter.x() + _R_ 
	    || m_x+m_width < canvas_rect::defCenter.x() - _R_
	    || m_y > canvas_rect::defCenter.y() + _R_
	    || m_y+m_height < canvas_rect::defCenter.y() - _R_  )
    {
	setFish(false);
    }
    else
    {
	setFish(true);
    }
}

void canvas_rect::printSize()
{
    kdWarning()<<"canvas rect size | width: "<<m_width<<" height: "<<m_height<<" x: "<<m_x<<" y: "<<m_y<<endl;
}



