/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _LPCANVAS_H
#define _LPCANVAS_H


#include <qcanvas.h>
#include <qcolor.h>
#include <qvaluelist.h>
#include "algo/rectangle.h"

#include <vector>

//using namespace std;

class canvas_rect;
class packingdata;
class QTimer;
class QCursor;
class KURL;
class QTextStream;
class QTimer;

class canvas_view : public QCanvasView 
{
    Q_OBJECT

    public:
	canvas_view(QWidget * parent, packingdata* );

	void initColors();

	void updateRectPos(int i);
	void clear();
	void update();

	void exportSVG(QTextStream*);
	void exportPNG(QString);

    protected:
	void resizeEvent(QResizeEvent*);
	void mousePressEvent(QMouseEvent*);
	void mouseDoubleClickEvent(QMouseEvent*);
	void contentsMouseMoveEvent(QMouseEvent*);
	void leaveEvent(QEvent*);
	void enterEvent(QEvent*);

    signals:
	void status(const QString&);
	void over(const QString &);
	void flip(int i);

    public slots:
        void check_rectangle();
	void updateRect(int i);
	void loadSettings();

    private:
	packingdata *m_data;

	QCanvasItem* m_moving;
	QPoint m_moving_current;
	int m_selectedRectangle;
	float m_ratio, m_xdecalage, m_ydecalage;

	bool m_updating;

	void checkAllFish();

	bool tryLock();
	void unlock();

	void updateRatio();
	void setRectColor(int);

	QValueList <QColor> m_couleursPlain;
	QValueList <QColor> m_couleursLight;
	QValueList <QColor> m_couleursGray;
	QValueList <QColor> m_couleursRedGreen;
	QValueList <canvas_rect*> m_rectlist;

	canvas_rect*  m_oldhover;

	QTimer *m_timer;
};

#endif // _LPCANVAS_

