/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qmutex.h>
#include <qlcdnumber.h>
#include <qtable.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qlabel.h>

#include <kdebug.h>
#include <kurl.h>
#include <kdebug.h>
#include <klocale.h>

#include "computationdata.h"
#include "datatable.h"
#include "packingdata.h"
#include "dataview.h"

#include "constants.h"


dataview::dataview(QWidget *parent, packingdata *data) : dataview_base(parent)
{
    m_data = data;

    connect(dataTable, SIGNAL(valueChanged(int, int)),
	    this, SLOT(changeDataRequested(int, int)) );

    heightLcd->setSegmentStyle(QLCDNumber::Flat);

    for (int i=0;i<6;i++)
	dataTable->setColumnWidth(i,75);

    for (int i=2;i<6;i++)
	dataTable->setColumnReadOnly(i, true);

    dataTable->setNumRows(TAILLETABLE);

    for (int i=0;i<TAILLETABLE;i++)
    {
	dataTable->setItem( i, 2, new dataTableItem( dataTable, QTableItem::Never ) );
	//dataTable->item( i, 2 )->setReplaceable(true);
	dataTable->setItem( i, 3, new dataTableItem( dataTable, QTableItem::Never ) );
	dataTable->setItem( i, 4, new dataTableItem( dataTable, QTableItem::Never ) );
	dataTable->setItem( i, 5, new dataTableItem( dataTable, QTableItem::Never ) );
    }
}

dataview::~dataview()
{

}

void dataview::update()
{
    for (int i=2;i<6;i++)
	dataTable->setColumnReadOnly(i, false);

    // DEBUG
    //kdWarning()<<"\033[1;33m dataview::update  \033[0m"<<endl;
    for (unsigned int i=0; i<m_data->nbRect(); i++)
    {
	if (m_data->isValidRect(i))
	{
	    updateRect(i);
	}
	else
	{
	    delete dataTable->item(i, 0);
	    delete dataTable->item(i, 1);

	    // the following code does not work :
	    /*delete dataTable->item(i, 2);
	      dataTable->setItem( i, 2, new dataTableItem( dataTable, QTableItem::Never ) );*/

	    if (dataTable->item(i, 2))
		dataTable->item(i, 2)->setText("");
	    if (dataTable->item(i, 3))
		dataTable->item(i, 3)->setText("");
	    if (dataTable->item(i, 4))
		dataTable->item(i, 4)->setText("");
	    if (dataTable->item(i, 5))
		dataTable->item(i, 5)->setText("");
	}
    }

    for (int i=2;i<6;i++)
	dataTable->setColumnReadOnly(i, true);

    heightLcd->display( m_data->Haut() );

    if ( m_data->isValidPacking() )
    {
	validLabel->setText( i18n("Packing is valid") );
	//kdWarning()<<"\033[1;32m valid packing \033[0m"<<endl;
    }
    else
    {
	//kdWarning()<<"\033[1;32m packing is NOT valid \033[0m"<<endl;	
	validLabel->setText( i18n("Packing is NOT valid") );
    }

    areaLabel->setText( "<p align=\"right\">"+QString::number(m_data->ratioArea())+"%"+"<p>" );
    largLabel->setText( "<p align=\"right\">"+QString::number(m_data->LB())+"<p>" );
    nbrectLabel->setText( "<p align=\"right\">"+QString::number(m_data->nbValidRect())+"<p>" );
}

void dataview::changeDataRequested(int i, int j)
{
    // DEBUG
    // kdWarning()<<"changeDataRequested"<<endl;

    switch (j)
    {
	case 0:
	    m_data->setWidth(i, dataTable->text(i, j).toInt() );
	    break;
	case 1:
	    m_data->setHeight(i, dataTable->text(i, j).toInt() );
	    break;

	default:
	    kdWarning()<<"BUG : dataview::changeDataRequested : not implemented"<<endl;
	    break;
    }
}

void dataview::loadSettings()
{
    update();
}

void dataview::updateRect(int i)
{
    dataTable->setText(i, 0, QString::number(  m_data->getRect(i).r_long)  );
    dataTable->setText(i, 1, QString::number(  m_data->getRect(i).r_larg)  );
    dataTable->setText(i, 2, QString::number(  m_data->getRect(i).posx)  );
    dataTable->setText(i, 3, QString::number(  m_data->getRect(i).posy)  );
    dataTable->setText(i, 4, QString::number(  (int) m_data->getRect(i).orientation )  );
    dataTable->setText(i, 5, QString::number(  m_data->getRect(i).numero)  );
}

#include "dataview.moc"

