/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _LINPACKER_H_
#define _LINPACKER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <dcopobject.h>
#include <dcopclient.h>

#include <kurl.h>
#include <kapplication.h>
#include <kmainwindow.h>

class KRecentFilesAction;
class KToggleAction;
//class KURL;

class algobib;
class packingdata;
class logview;
class dataview;
class canvas_view;
class algorun;
/**
 * This class serves as the main window for linpacker.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Thomas Nagy <tnagy256@yahoo.fr>
 * @version 0.5.3
 */
class linpacker : public KMainWindow, public DCOPObject
{
    Q_OBJECT
    K_DCOP

public:
    linpacker();
    virtual ~linpacker();
    void updateAllViews();
	
public slots:
    void fileOpen();
    void fileQuit();
    void fileSave();
    void fileSaveAs();

    void openURL(const KURL & url);

    void launch();
    void exportData();
    void manageTable();

    void optionsPreferences();
    void optionsAlgorithms();
    
    void setStaticMessage(const QString&);
    void setTemporaryMessage(const QString &);
	
private:
    void setupAccel();
    void setupActions();

protected:
    virtual void dragEnterEvent(QDragEnterEvent*);
    virtual void dropEvent(QDropEvent*);
    virtual bool queryClose();
    
private:
    dataview      * m_dataview;
    canvas_view   * m_canvasview;
    logview       * m_logview;
    packingdata   * m_data;

    algorun       * m_algorun;

    bool m_locked;

    KRecentFilesAction * m_openRecentAction;
    
    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;
    KURL m_url;

k_dcop:
	void start_solver(); // same as (slot) launch, but for dcop
};

#endif // _LINPACKER_H_

