/***************************************************************************
 *   Copyright (C) 2004 by Thomas Nagy                                     *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <qmutex.h>
#include <qstring.h>

#include <kdebug.h>

#include "computationdata.h"
#include "packingdata.h"
#include "lnpkreader.h"

LNPKReader::LNPKReader(packingdata *p) : QXmlDefaultHandler()
{
    m_data=p;
    m_nbrect=0;
}

bool LNPKReader::startDocument()
{
    return true;
}

bool LNPKReader::startElement( const QString&, const QString&, const QString& qName, const QXmlAttributes& attribs)
{
    // DEBUG	
    //kdWarning()<<"qname "<<qName<<endl;

    if (qName == "log_algorithm")
	m_data->m_adata.m_algoname = attribs.value(0);
    else if (qName == "log_bandeheight")
	m_data->m_cdata.m_height = attribs.value(0).toInt();
    else if (qName == "log_bandewidth")
	m_data->m_cdata.m_containerLB = attribs.value(0).toInt();
    else if (qName == "log_nbrect")
	m_data->m_cdata.m_nbrect = attribs.value(0).toInt();
    else if (qName == "log_arearatio")
	m_data->m_cdata.m_arearatio = attribs.value(0).toInt();
    else if (qName == "log_totaltime")
	m_data->m_cdata.m_totaltime = attribs.value(0).toInt();
    else if (qName == "log_textlog")
	m_data->m_cdata.m_textlog = attribs.value(0);
    else if (qName == "log_nbiter")
	m_data->m_adata.m_iter = attribs.value(0).toInt();
    else if (qName == "log_nbelements")
	m_data->m_adata.m_population = attribs.value(0).toInt();
    else if (qName == "log_mutations")
	m_data->m_adata.m_mutations = attribs.value(0).toInt();
    else if (qName == "log_crossovers")
	m_data->m_adata.m_crossovers = attribs.value(0).toInt();
    else if (qName == "log_fitnesstype")
	m_data->m_adata.m_fitnesstype = attribs.value(0).toInt();

    // object attribs
    else if (qName == "bandewidth")
	m_data->setLB( attribs.value(0).toInt() );
    else if (qName == "width")
	m_data->setWidth( m_nbrect, attribs.value(0).toInt() );
    else if (qName == "height")
	m_data->setHeight( m_nbrect, attribs.value(0).toInt() );
    else if (qName == "posx")
	m_data->m_vrect[m_nbrect].posx = attribs.value(0).toInt();
    else if (qName == "posy")
	m_data->m_vrect[m_nbrect].posy = attribs.value(0).toInt();
    else if (qName == "orientation")
	m_data->m_vrect[m_nbrect].orientation = (bool) attribs.value(0).toInt();
    else if (qName == "order")
	m_data->m_vrect[m_nbrect].numero = attribs.value(0).toInt();
    else if (qName == "num")
    {
	if (attribs.value(0).toInt() != m_nbrect)
	{
	    kdWarning()<<"old or invalid file used : "<<m_nbrect<<" "<<attribs.value(0).toInt()<<endl;
	}
    }
    else if (qName == "name")
	m_data->m_vrect[m_nbrect].name = attribs.value(0);

    return true;
}

bool LNPKReader::endElement( const QString&, const QString&, const QString& qName )
{
    if (qName == "object" || qName == "rect")
	m_nbrect++;

    return true;
}


