/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include <kdebug.h>
#include <kurl.h>
#include <ktempfile.h>
#include <kio/netaccess.h>
#include <kmessagebox.h>

#include <unistd.h>

#include "constants.h"

#include "computationdata.h"
#include "packingdata.h"
#include "sys/time.h"
#include "algo/algobase.h"

#include "gui/settings.h"
#include "lnpkreader.h"


static packingdata* data = NULL;

packingdata* packingdata::getData()
{
    if (!data)
	data = new packingdata();
    return data;
}

packingdata::packingdata() : QObject()
{
    m_containerWidth = 10000;
    m_vrect.resize( NBRECT );

    resetPacking();

    // DEBUG
    //m_vrect[0] = rectangle(2000, 1000, 0, 0, true, 0);
    //m_vrect[1] = rectangle(1000, 6000, 2000, 2000, true, 1);

    m_computing = false;
    m_ischanged = false;
}

void packingdata::resetPacking()
{
    rectangle nullrect(0, 0, 0, 0, false, 0);
    for (int i=0;i<nbRect();i++)
	setRect(i, nullrect);

    m_ischanged = true;
}

int packingdata::LB() const
{
    return m_containerWidth;	
}

int packingdata::Haut() const
{
    int val = 0;
    for (unsigned int i=0;i<m_vrect.size();i++)
    {
	if (isValidRect(i))
	    if (val < m_vrect[i].YHaut()) 
		val = m_vrect[i].YHaut();
    }
    return val;
}

void packingdata::setLB(int lb)
{
    m_ischanged = true;
    if (lb>100 and lb<100000)
	m_containerWidth = lb;
    else
	m_containerWidth = 10000;
}

int packingdata::nbRect() const
{
    return (int) m_vrect.size();
}

rectangle packingdata::getRect(int i) const
{
    if (i>= 0 and i < (int) m_vrect.size() )
	return m_vrect[i];
    else
    {
	kdWarning()<<"BUG : packingdata::getRect"<<endl;
	return rectangle(0,0,0,0,0,0);
    }
}

//
int packingdata::ratioArea() const
{
    //kdWarning()<<"packingdata::ratioArea"<<endl;

    if (! isValidPacking())
	return 0;

    long double ssurf = 0;
    for (unsigned int i=0;i<m_vrect.size();i++)
    {
	if (isValidRect(i))
	    ssurf+=m_vrect[i].Surface();
    }

    if (ssurf==0)
	return 0;

    long double stot = (double) Haut() * (double) m_containerWidth;
    long double thing = (100 * ssurf) / (stot);


    //kdWarning()<<"ratio area : "<<(int) thing<<endl;

    return (int) (thing);
}

// is the problem worth solving ?
bool packingdata::pbvalide() const
{
    if (nbValidRect() < 1)
	return false;

    for (unsigned int z = 0; z < m_vrect.size(); z++)
	if (m_vrect[z].largeur() > m_containerWidth && m_vrect[z].longueur() > m_containerWidth)
	    return false;

    return true;
}

// helper function
QString printvalue(const QString & tagname, int value)
{
    return QString("      <")+tagname+QString(" value=\"")+QString::number(value)+QString("\" />\n");
}

// helper function
QString printvaluestring(const QString & tagname, const QString & str)
{
    return QString("      <")+tagname+QString(" value=\"")+str+QString("\" />\n");
}

bool packingdata::saveFile(KURL url)
{
    QString fname = url.url();

    if (fname.isEmpty())
    {
	kdWarning()<<"empty file !!"<<endl;
	return false;
    }

    KTempFile docfile;
    docfile.setAutoDelete(true);

    QTextStream *s = docfile.textStream();

    if (fname.endsWith(".lnpk"))
    {
	*s<<QString("<?xml version=\"1.0\" encoding=\"utf8\"?>\n");
	*s<<"<project>\n";
	*s<<"   <settings>\n";
	*s<<printvalue("bandewidth", LB() );

	*s<<printvaluestring("log_algorithm", m_adata.m_algoname);
	*s<<printvalue("log_bandeheight", m_cdata.m_height);
	*s<<printvalue("log_bandewidth", m_cdata.m_containerLB);
	*s<<printvalue("log_nbrect", m_cdata.m_nbrect);
	*s<<printvalue("log_arearatio", m_cdata.m_arearatio);
	*s<<printvalue("log_totaltime", m_cdata.m_totaltime);
	*s<<printvaluestring("log_textlog", m_cdata.m_textlog);
	*s<<printvalue("log_nbiter", m_adata.m_iter);
	*s<<printvalue("log_nbelements", m_adata.m_population);
	*s<<printvalue("log_mutations", m_adata.m_mutations);
	*s<<printvalue("log_crossovers", m_adata.m_crossovers);
	*s<<printvalue("log_maxtime", m_adata.m_maxtime);
	*s<<printvalue("log_fitnesstype", m_adata.m_fitnesstype);
	*s<<"   </settings>\n";	

	*s<<"   <objects>\n";

	for (int i = 0; i < nbRect(); i++)
	{
	    if (!isValidRect(i))
		continue;

	    *s<<"     <rect>\n";

	    *s<<printvalue("width", m_vrect[i].r_larg);
	    *s<<printvalue("height", m_vrect[i].r_long);
	    *s<<printvalue("posx", m_vrect[i].posx);
	    *s<<printvalue("posy", m_vrect[i].posy);
	    *s<<printvalue("orientation", m_vrect[i].orientation);
	    *s<<printvalue("order", m_vrect[i].numero);
	    *s<<printvalue("num", i);
	    *s<<printvaluestring("name", m_vrect[i].name);

	    *s<<"     </rect>\n";
	}

	*s<<"   </objects>\n";
	*s<<"</project>\n";
    }

    docfile.close();

    if ( KIO::NetAccess::upload(docfile.name(), url, NULL) )
    {
	m_ischanged = false;
	return true;
    }
    else
    {
	return false;
    }
}

bool packingdata::openFile(KURL url)
{
    m_computing=true;

    QString tmpFile;
    if (! KIO::NetAccess::download(url, tmpFile, NULL))
    {
	QString text;

	if ( url.isLocalFile() )
	    text = i18n( "Unable to find the linpacker file %1 !" );
	else
	    text = i18n( "Unable to download the linpacker file ! "
		    "Please check that the address %1 is correct." );

	KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );

	m_computing=false;
	return false;
    }

    QFile file(tmpFile);
    LNPKReader handler(this);
    QXmlInputSource source( &file );
    QXmlSimpleReader reader;
    reader.setContentHandler( &handler );

    if (! reader.parse( source ))
    {
	QString text;
	text = i18n( "Could not parse the file %1 entirely - some data may be inconsistent" );

	KMessageBox::sorry( NULL, text.arg( url.prettyURL() )  );

	KIO::NetAccess::removeTempFile( tmpFile );

	resetPacking();

	m_computing=false;
	return false;
    }

    m_computing=false;
    return true;
}

void packingdata::setRect(int idx, rectangle rect)
{
    m_vrect[idx] = rect;
    m_ischanged = true;

    if (!m_computing)
    {
	//kdWarning()<<"signal rectangle changed emitted"<<endl;
	emit rectangle_changed(idx);
    }
}

void packingdata::setWidth(int idx, int val)
{
    if (idx > NBRECT || idx <0)
    {
	kdWarning()<<"BUG : packingdata::setWidth of idx: "<<idx<<" and value: "<<val<<endl;
	return;
    }
    m_ischanged=true;
    m_vrect[idx].r_larg = val;

    //kdWarning()<<"packingdata::setWidth rectangle changed "<<idx<<endl;

    if (!m_computing)
    {
	//kdWarning()<<"signal rectangle changed emitted"<<endl;
	emit rectangle_changed(idx);
    }
}

void packingdata::setHeight(int idx, int val)
{
    if (idx > NBRECT || idx <0)
    {
	kdWarning()<<"BUG : packingdata::setHeight of idx: "<<idx<<" and value: "<<val<<endl;
	return;
    }
    m_ischanged=true;
    m_vrect[idx].r_long = val;

    //kdWarning()<<"packingdata::setHeight rectangle changed "<<idx<<endl;

    if (!m_computing)
    {
	//kdWarning()<<"signal rectangle changed emitted"<<endl;
	emit rectangle_changed(idx);
    }
}

int packingdata::width(int i) const
{
    return m_vrect[i].r_larg;
}

int packingdata::height(int i) const
{
    return m_vrect[i].r_long;
}

int packingdata::posX(int i) const
{
    return m_vrect[i].posx;
}

int packingdata::posY(int i) const
{
    return m_vrect[i].posy;
}

bool packingdata::isValidRect(int i) const
{
    if (i < 0 || i >= nbRect() )
	return false;

    rectangle r = m_vrect[i];
    if (r.r_larg <= 0 || r.r_larg > RECTMAXSIZE)
	return false;
    if (r.r_long <= 0 || r.r_long > RECTMAXSIZE)
	return false;

    return true;
}

bool packingdata::isValidPacking() const
{
    if (nbRect() == 0)
	return true;

    for (int i = 0; i < nbRect(); i++)
    {
	if (isValidRect(i))
	{
	    for (int j = i+1; j < nbRect(); j++)
	    {
		if (isValidRect(j))
		{
		    if (rectangle::overlapping(m_vrect[i], m_vrect[j]) )
			return false;
		}
	    }
	}
    }
    
    return true;
}

int packingdata::nbValidRect() const
{
    int nbvalid = 0;
    for (int i = 0; i < nbRect(); i++)
    {
	if (isValidRect(i))
	    nbvalid++;
    }
    return nbvalid;
}

// helper function
QString snum(int i)
{
    return QString::number(i);
}

void packingdata::exportCSV(QTextStream* s)
{
    *s<<"# linpacker generated file\n";
    *s<<"# ";

    *s<<"\""<<m_adata.m_algoname<<"\" ";
    *s<<snum(m_cdata.m_height)<<" ";
    *s<<snum(m_cdata.m_containerLB)<<" ";
    *s<<snum(m_cdata.m_nbrect)<<" ";
    *s<<snum(m_cdata.m_arearatio)<<" ";
    *s<<snum(m_cdata.m_totaltime)<<" ";
    *s<<snum(m_adata.m_iter)<<" ";
    *s<<snum(m_adata.m_population)<<" ";
    *s<<snum(m_adata.m_mutations)<<" ";
    *s<<snum(m_adata.m_crossovers)<<" ";
    *s<<snum(m_adata.m_maxtime)<<" ";
    *s<<snum(m_adata.m_fitnesstype);
    *s<<"\n";

    for (int i = 0; i < nbRect(); i++)
    {
	if (!isValidRect(i))
	    continue;

	*s<<snum(m_vrect[i].r_larg)<<" ";
	*s<<snum(m_vrect[i].r_long)<<" ";
	*s<<snum(m_vrect[i].posx)<<" ";
	*s<<snum(m_vrect[i].posy)<<" ";
	*s<<snum(m_vrect[i].orientation)<<" ";
	*s<<snum(m_vrect[i].numero)<<" ";
	*s<<snum(i)<<" ";
	*s<<"\""<<m_vrect[i].name<<"\"";
	*s<<"\n";
    }
}

void packingdata::makePackingValid()
{
    int top = 0;
    int numrect = 0;
    for (int i=0; i<nbRect(); i++)
    {
	//kdWarning()<<"packingdata::makePackingValid "<<i<<endl;
	if (!isValidRect(i))
	    continue;
	
	m_vrect[i].posx = 0;
	m_vrect[i].posy = top;
	m_vrect[i].numero = numrect;
	top += m_vrect[i].longueur();
	emit rectangle_changed(i);
	numrect++;
    }
}

#include "packingdata.moc"

