/***************************************************************************
 *   Copyright (C) 2003, 2004 by Thomas Nagy                               *
 *   tnagy256@yahoo.fr                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _PACKINGDATA_H
#define _PACKINGDATA_H

#include <vector>
using namespace std;

#include <qobject.h>
#include <qtextstream.h>

#include <kurl.h>

#include "algo/rectangle.h"

struct computationdata;
struct algodata;
class QMutex;
class packingdata : public QObject
{
	Q_OBJECT
	
	friend class LNPKReader; // class used for loading data ..
	
	public:
		static packingdata* getData();
	

		computationdata m_cdata;
		algodata        m_adata;
		
		// may be useful elsewhere - clears all data
		void resetPacking();
		
		//////////////////////////////////////
		//  rectangle-related
		int nbRect() const;
		int nbValidRect() const;
		
		rectangle getRect(int) const; // returns a copy 
		bool isValidRect(int) const;

		///////////////////////////////////////
		// accessors for the rectangle table
		void setRect(int idx, rectangle rect);
		void setWidth(int idx, int val);
		void setHeight(int idx, int val);
		
		int width(int idx) const;
		int height(int idx) const;
		int posX(int idx) const;
		int posY(int idx) const;

		////////////////////////////////////////
		// packing-related
		
		int	Haut() const; // height of the packing
		
		int	LB() const; // size of the container
		void setLB(int);

		bool isValidPacking() const;
		int	ratioArea() const;

		////////////////////////////////////////
		// input/output
		bool saveFile(KURL);
		bool openFile(KURL);
		void exportCSV(QTextStream*);

		bool pbvalide() const;

		bool m_computing;
		int m_status;

		bool m_ischanged;


		/////////////////////////////////////////
		// misc
		
		void makePackingValid(); // boxmanager
		
	signals:
		void rectangle_changed(int i);
		
	private:
		packingdata();
		int m_containerWidth; //largeurbande;
		vector<rectangle> m_vrect;

};


#endif // _PACKINGDATA_H

